/*
 * DO NOT ALTER OR REMOVE COPYRIGHT NOTICES OR THIS HEADER.
 *
 * The contents of this file are subject to the terms of either the GNU
 * General Public License Version 2 only ("GPL") or the Common
 * Development and Distribution License("CDDL") (collectively, the
 * "License"). You may not use this file except in compliance with the
 * License. You can obtain a copy of the License at
 * http://www.netbeans.org/cddl-gplv2.html
 * or nbbuild/licenses/CDDL-GPL-2-CP. See the License for the
 * specific language governing permissions and limitations under the
 * License.  When distributing the software, include this License Header
 * Notice in each file and include the License file at
 * nbbuild/licenses/CDDL-GPL-2-CP.  Sun designates this
 * particular file as subject to the "Classpath" exception as provided
 * by Sun in the GPL Version 2 section of the License file that
 * accompanied this code. If applicable, add the following below the
 * License Header, with the fields enclosed by brackets [] replaced by
 * your own identifying information:
 * "Portions Copyrighted [year] [name of copyright owner]"
 *
 * Contributor(s):
 *
 * Portions Copyrighted 2009 Jaroslav Tulach
 */

package cz.xelfi.quoridor.webidor;

import com.sun.jersey.test.framework.JerseyTest;
import java.io.File;
import java.io.FileOutputStream;
import java.io.IOException;
import javax.ws.rs.core.MediaType;
import org.junit.Test;
import static org.junit.Assert.*;

/**
 *
 * @author Jaroslav Tulach <jtulach@netbeans.org>
 */
public class FinishedGameTest extends JerseyTest {
    private File dir;

    public FinishedGameTest() throws Exception {
        super("cz.xelfi.quoridor.webidor.resources");
    }

    @Override
    public void setUp() throws Exception {
        dir = File.createTempFile("quoridor", ".dir");
        dir.delete();
        System.setProperty("quoridor.dir", dir.getPath());
        dir.mkdirs();
        File passwd = new File(dir, "passwd");
        FileOutputStream os = new FileOutputStream(passwd);
        os.write("Jarda=heslo\nJirka=pesko\n".getBytes("UTF-8"));
        os.close();
        super.setUp();
    }

    @Override
    public void tearDown() throws Exception {
        super.tearDown();
        deleteRec(dir);
    }

    static void deleteRec(File dir) throws IOException {
        if (dir == null) {
            return;
        }
        File[] arr = dir.listFiles();
        if (arr != null) {
            for (File f : arr) {
                deleteRec(f);
            }
        }
        dir.delete();
    }
    @Test public void testNotLoggedIn() {
        String status  = webResource.path("login").queryParam("id", "not-logged-in").
                accept(MediaType.TEXT_PLAIN).get(String.class);
        assertEquals("Nobody is logged in", "", status);
    }


    @Test public void testCreateAGame() throws Exception {
        String logJarda = webResource.path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        String logJirka = webResource.path("login").
            queryParam("name", "Jirka").
            queryParam("password", "pesko").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        assertNotNull("Logged in ok", logJarda);
        GameId s = webResource.path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);

        for (int i = 0; i < 3; i++) {
            webResource.path("games/" + s.getId())
                .queryParam("loginID", logJarda)
                .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
            webResource.path("games/" + s.getId())
                .queryParam("loginID", logJirka)
                .queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        }

        webResource.path("games/" + s.getId())
            .queryParam("loginID", logJarda)
            .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
        webResource.path("games/" + s.getId())
            .queryParam("loginID", logJirka)
            .queryParam("player", "Jirka").queryParam("move", "SS").put(GameId.class);

        for (int i = 0; i < 3; i++) {
            webResource.path("games/" + s.getId())
                .queryParam("loginID", logJarda)
                .queryParam("player", "Jarda").queryParam("move", "N").put(GameId.class);
            webResource.path("games/" + s.getId())
                .queryParam("loginID", logJirka)
                .queryParam("player", "Jirka").queryParam("move", "S").put(GameId.class);
        }

        Game end = webResource.path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("BlackWins", GameStatus.blackWon, end.getId().getStatus());
    }

    @Test public void testResignAGame() throws Exception {
        String logJarda = webResource.path("login").
            queryParam("name", "Jarda").
            queryParam("password", "heslo").
            accept(MediaType.TEXT_PLAIN).
            put(String.class);
        GameId s = webResource.path("games").queryParam("white", "Jarda")
                .queryParam("loginID", logJarda)
                .queryParam("black", "Jirka").post(GameId.class);

        webResource.path("games/" + s.getId()).
            queryParam("loginID", logJarda).
            queryParam("player", "Jarda").
            queryParam("move", "RESIGN").put(GameId.class);
        Game end = webResource.path("games/" + s.getId()).accept(MediaType.TEXT_XML).get(Game.class);
        assertEquals("BlackWins", GameStatus.blackWon, end.getId().getStatus());
    }

}
